"""Middlewares package.

Example middleware:

```python
from meator.middlewares import Middleware

class CustomMiddleware(Middleware):
    async def __call__(self, request: Request) -> tp.Any:
        ...
        result = await self.call_next(request)
        ...
        return result
```
"""

from .base import Middleware
from .wrap_handler import wrap_handler

__all__ = ("Middleware", "wrap_handler")



import typing as tp

from meator.entities.request import Request
from meator.interfaces.middleware import IMiddleware
from meator.interfaces.types import CallNextType

Res = tp.TypeVar("Res")


class Middleware(IMiddleware):
    @tp.final
    def set_next(self, call_next: CallNextType) -> tp.Self:
        self.call_next = call_next
        return self

    async def __call__(self, request: Request) -> tp.Any:
        return await self.call_next(request)



import typing as tp

from meator.entities import Request
from meator.interfaces.handlers import IHandler
from meator.interfaces.types import CallNextType


class IMiddleware(IHandler[Request, tp.Any]):
    call_next: CallNextType



import typing as tp
from functools import reduce

from meator.interfaces.handlers.request import IHandler
from meator.interfaces.types import CallNextType
from meator.middlewares.base import Middleware


def wrap_handler(middlewares: tp.Sequence[Middleware], handler: IHandler) -> CallNextType:
    return reduce(lambda x, y: y.set_next(x), middlewares, handler)
